@@ -1,6 +1,34 @@
-# file MASS/add.q
-# copyright (C) 1994-2006 W. N. Venables and B. D. Ripley
+# file MASS/R/add.R
+# copyright (C) 1994-2008 W. N. Venables and B. D. Ripley
 #
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 or 3 of the License
+#  (at your option).
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+#
+
+## version to return NA for df = 0, as R did before 2.7.0
+safe_pchisq <- function(q, df, ...)
+{
+    df[df <= 0] <- NA
+    pchisq(q=q, df=df, ...)
+}
+## and to avoid a warning
+safe_pf <- function(q, df1, ...)
+{
+    df1[df1 <= 0] <- NA
+    pf(q=q, df1=df1, ...)
+}
+
+
 addterm <-
     function(object, ...) UseMethod("addterm")
 
@@ -22,15 +50,16 @@ addterm.default <-
                   dimnames = list(c("<none>", scope), c("df", "AIC")))
     ans[1,  ] <- extractAIC(object, scale, k = k, ...)
     n0 <- length(object$residuals)
+    env <- environment(formula(object))
     for(i in seq(ns)) {
         tt <- scope[i]
         if(trace) {
-	    cat("trying +", tt, "\n")
-	    flush.console()
+	    message("trying +", tt)
+	    utils::flush.console()
         }
         nfit <- update(object, as.formula(paste("~ . +", tt)),
                        evaluate = FALSE)
-        nfit <- eval.parent(nfit)
+	nfit <- eval(nfit, envir=env) # was  eval.parent(nfit)
 	ans[i+1, ] <- extractAIC(nfit, scale, k = k, ...)
         if(length(nfit$residuals) != n0)
             stop("number of rows in use has changed: remove missing values?")
@@ -38,14 +67,14 @@ addterm.default <-
     dfs <- ans[,1] - ans[1,1]
     dfs[1] <- NA
     aod <- data.frame(Df = dfs, AIC = ans[,2])
-    o <- if(sorted) order(aod$AIC) else seq(along=aod$AIC)
+    o <- if(sorted) order(aod$AIC) else seq_along(aod$AIC)
     test <- match.arg(test)
     if(test == "Chisq") {
 	dev <- ans[,2] - k*ans[, 1]
 	dev <- dev[1] - dev; dev[1] <- NA
 	nas <- !is.na(dev)
 	P <- dev
-	P[nas] <- pchisq(dev[nas], dfs[nas], lower.tail=FALSE)
+	P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail=FALSE)
 	aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
     }
     aod <- aod[o, ]
@@ -81,7 +110,7 @@ addterm.lm <-
     if(scale > 0) aic <- RSS/scale - n + k*dfs
     else aic <- n * log(RSS/n) + k*dfs
     aod$AIC <- aic
-    o <- if(sorted) order(aod$AIC) else seq(along=aod$AIC)
+    o <- if(sorted) order(aod$AIC) else seq_along(aod$AIC)
     if(scale > 0) names(aod) <- c("Df", "Sum of Sq", "RSS", "Cp")
     test <- match.arg(test)
     if(test == "Chisq") {
@@ -93,10 +122,10 @@ addterm.lm <-
         } else dev <- dev/scale
         df <- aod$Df
         nas <- !is.na(df)
-        dev[nas] <- pchisq(dev[nas], df[nas], lower.tail=FALSE)
+        dev[nas] <- safe_pchisq(dev[nas], df[nas], lower.tail=FALSE)
         aod[, "Pr(Chi)"] <- dev
     } else if(test == "F") {
-        rdf <- object$df.resid
+        rdf <- object$df.residual
         aod[, c("F Value", "Pr(F)")] <- Fstat(aod, aod$RSS[1], rdf)
     }
     aod <- aod[o, ]
@@ -124,7 +153,7 @@ addterm.glm <-
 	Fs[df < .Machine$double.eps] <- NA
 	P <- Fs
 	nnas <- !is.na(Fs)
-	P[nnas] <- pf(Fs[nnas], df[nnas], rdf - df[nnas], lower.tail=FALSE)
+	P[nnas] <- safe_pf(Fs[nnas], df[nnas], rdf - df[nnas], lower.tail=FALSE)
 	list(Fs=Fs, P=P)
     }
     if(missing(scope) || is.null(scope)) stop("no terms in scope")
@@ -171,8 +200,8 @@ addterm.glm <-
                      function(x) paste(sort(x), collapse=":"))
     for(tt in scope) {
         if(trace) {
-	    cat("trying +", tt, "\n")
-	    flush.console()
+	    message("trying +", tt)
+	    utils::flush.console()
 	}
         stt <- paste(sort(strsplit(tt, ":")[[1]]), collapse=":")
 	usex <- match(asgn, match(stt, sTerms), 0) > 0
@@ -196,7 +225,7 @@ addterm.glm <-
     dfs[1] <- NA
     aod <- data.frame(Df = dfs, Deviance = dev, AIC = aic,
                       row.names = names(dfs), check.names = FALSE)
-    o <- if(sorted) order(aod$AIC) else seq(along=aod$AIC)
+    o <- if(sorted) order(aod$AIC) else seq_along(aod$AIC)
     if(all(is.na(aic))) aod <- aod[, -3]
     test <- match.arg(test)
     if(test == "Chisq") {
@@ -205,7 +234,7 @@ addterm.glm <-
         LRT <- if(dispersion == 1) "LRT" else "scaled dev."
         aod[, LRT] <- dev
         nas <- !is.na(dev)
-        dev[nas] <- pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
+        dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
         aod[, "Pr(Chi)"] <- dev
     } else if(test == "F") {
         if(fam == "binomial" || fam == "poisson")
@@ -245,16 +274,17 @@ dropterm.default <-
     ans <- matrix(nrow = ns + 1, ncol = 2,
                   dimnames =  list(c("<none>", scope), c("df", "AIC")))
     ans[1,  ] <- extractAIC(object, scale, k = k, ...)
+    env <- environment(formula(object))
     n0 <- length(object$residuals)
     for(i in seq(ns)) {
         tt <- scope[i]
         if(trace) {
-	    cat("trying -", tt, "\n")
-	    flush.console()
+	    message("trying -", tt)
+	    utils::flush.console()
 	}
         nfit <- update(object, as.formula(paste("~ . -", tt)),
                        evaluate = FALSE)
-        nfit <- eval.parent(nfit)
+	nfit <- eval(nfit, envir=env) # was  eval.parent(nfit)
 	ans[i+1, ] <- extractAIC(nfit, scale, k = k, ...)
         if(length(nfit$residuals) != n0)
             stop("number of rows in use has changed: remove missing values?")
@@ -262,14 +292,14 @@ dropterm.default <-
     dfs <- ans[1,1] - ans[,1]
     dfs[1] <- NA
     aod <- data.frame(Df = dfs, AIC = ans[,2])
-    o <- if(sorted) order(aod$AIC) else seq(along=aod$AIC)
+    o <- if(sorted) order(aod$AIC) else seq_along(aod$AIC)
     test <- match.arg(test)
     if(test == "Chisq") {
         dev <- ans[, 2] - k*ans[, 1]
         dev <- dev - dev[1] ; dev[1] <- NA
         nas <- !is.na(dev)
         P <- dev
-        P[nas] <- pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
+        P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
         aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
     }
     aod <- aod[o, ]
@@ -291,24 +321,24 @@ dropterm.lm <-
     n <- length(object$residuals)
     aod$AIC <- if(scale > 0)RSS/scale - n + k*dfs
     else n * log(RSS/n) + k*dfs
-    o <- if(sorted) order(aod$AIC) else seq(along=aod$AIC)
+    o <- if(sorted) order(aod$AIC) else seq_along(aod$AIC)
     if(scale > 0) names(aod) <- c("Df", "Sum of Sq", "RSS", "Cp")
     test <- match.arg(test)
     if(test == "Chisq") {
         dev <- aod$"Sum of Sq"
         nas <- !is.na(dev)
-        dev[nas] <- pchisq(dev[nas]/scale, aod$Df[nas], lower.tail = FALSE)
+        dev[nas] <- safe_pchisq(dev[nas]/scale, aod$Df[nas], lower.tail = FALSE)
         aod[, "Pr(Chi)"] <- dev
     } else if(test == "F") {
 	dev <- aod$"Sum of Sq"
 	dfs <- aod$Df
-	rdf <- object$df.resid
+	rdf <- object$df.residual
 	rms <- aod$RSS[1]/rdf
 	Fs <- (dev/dfs)/rms
 	Fs[dfs < 1e-4] <- NA
 	P <- Fs
 	nas <- !is.na(Fs)
-	P[nas] <- pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
+	P[nas] <- safe_pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
         aod[, c("F Value", "Pr(F)")] <- list(Fs, P)
     }
     aod <- aod[o, ]
@@ -341,7 +371,7 @@ dropterm.glm <-
   }
     ns <- length(scope)
     ndrop <- match(scope, tl)
-    rdf <- object$df.resid
+    rdf <- object$df.residual
     chisq <- object$deviance
     dfs <- numeric(ns)
     dev <- numeric(ns)
@@ -351,10 +381,10 @@ dropterm.glm <-
     if(is.null(wt)) wt <- rep.int(1, n)
     for(i in 1:ns) {
         if(trace) {
-	    cat("trying -", scope[i], "\n")
-	    flush.console()
+	    message("trying -", scope[i])
+	    utils::flush.console()
 	}
-        ii <- seq(along=asgn)[asgn == ndrop[i]]
+        ii <- seq_along(asgn)[asgn == ndrop[i]]
         jj <- setdiff(seq(ncol(x)), ii)
         z <-  glm.fit(x[, jj, drop = FALSE], y, wt, offset=object$offset,
                       family=object$family, control=object$control)
@@ -378,7 +408,7 @@ dropterm.glm <-
     aic <- aic + (extractAIC(object, k = k)[2] - aic[1])
     aod <- data.frame(Df = dfs, Deviance = dev, AIC = aic,
                       row.names = scope, check.names = FALSE)
-    o <- if(sorted) order(aod$AIC) else seq(along=aod$AIC)
+    o <- if(sorted) order(aod$AIC) else seq_along(aod$AIC)
     if(all(is.na(aic))) aod <- aod[, -3]
     test <- match.arg(test)
     if(test == "Chisq") {
@@ -387,11 +417,11 @@ dropterm.glm <-
         nas <- !is.na(dev)
         LRT <- if(dispersion == 1) "LRT" else "scaled dev."
         aod[, LRT] <- dev
-        dev[nas] <- pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
+        dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
         aod[, "Pr(Chi)"] <- dev
     } else if(test == "F") {
         if(fam == "binomial" || fam == "poisson")
-            warning(gettextf("F test assumes quasi%s family", fam),
+            warning(gettextf("F test assumes 'quasi%s' family", fam),
                     domain = NA)
 	dev <- aod$Deviance
 	rms <- dev[1]/rdf
@@ -402,7 +432,7 @@ dropterm.glm <-
 	Fs[dfs < 1e-4] <- NA
 	P <- Fs
 	nas <- !is.na(Fs)
-	P[nas] <- pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
+	P[nas] <- safe_pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
 	aod[, c("F value", "Pr(F)")] <- list(Fs, P)
     }
     aod <- aod[o, ]
